#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['wheel', 'wheel._commands']

package_data = \
{'': ['*']}

package_dir = \
{'': 'src'}

install_requires = \
['packaging >= 24.0']

entry_points = \
{'console_scripts': ['wheel = wheel._commands:main'],
 'distutils.commands': ['bdist_wheel = wheel.bdist_wheel:bdist_wheel']}

setup(name='wheel',
      version='0.47.0',
      description='Command line tool for manipulating wheel files',
      author=None,
      author_email='Daniel Holth <dholth@fastmail.fm>',
      url=None,
      packages=packages,
      package_data=package_data,
      package_dir=package_dir,
      install_requires=install_requires,
      entry_points=entry_points,
      python_requires='>=3.9',
     )
